<?php

namespace Drupal\Tests\entity_access_by_reference_field\Functional;

use Drupal\entity_access_by_reference_field\Helper\Constants;
use Drupal\user\Entity\Role;

/**
 * This class provides access tests for entity_access_by_reference_field.
 *
 * @group entity_access_by_reference_field
 */
class EntityAccessFallbackTest extends EntityAccessTestBase {

  /**
   * {@inheritDoc}
   */
  protected function setUp(): void {
    parent::setUp();
    $this->createContentType(['type' => 'notice']);
  }

  /**
   * Setup fallback tests.
   */
  public function setupFallbackTests() {
    $this->createEntityReferenceField('node', 'article', 'field_notice_test', 'field_notice_test', 'node', 'default', ['target_bundles' => ['notice']]);
    $this->fieldStorage = \Drupal::entityTypeManager()->getStorage('field_storage_config')->load('node.field_notice_test');

    // Create a notice:
    $referencedNode = $this->drupalCreateNode([
      'id' => 1,
      'title' => 'Notice',
      'type' => 'notice',
      'body' => 'Body field value.',
    ]);
    $referencedNode->save();

    // Create an article referencing the notice:
    $node = $this->drupalCreateNode([
      'id' => 2,
      'title' => 'My Test Node',
      'type' => 'article',
      'body' => 'Body field value.',
      'field_notice_test' => [
        0 => [
          'target_id' => $referencedNode->id(),
        ],
      ],
    ]);
    $node->save();
  }

  /**
   * Test fallback neutral.
   */
  public function testFallbackNeutral() {
    $this->drupalLogin($this->authenticatedUser);
    $this->setupFallbackTests();

    // Allow all $permissionMatrixValues:
    $permissionMatrixValues = $this->initiatePermissionMatrix();

    // Give the permission to update the entity if the user is allowed to
    // update the referenced entity:
    $this->setupThirdPartySettings($this->fieldStorage, TRUE, $permissionMatrixValues, Constants::FALLBACK_NEUTRAL);

    // Check if the edit page is not accessible:
    $this->drupalGet('/node/2/edit');
    $this->assertSession()->statusCodeEquals(403);

    // Grant permission to edit the referenced article:
    $authenticatedRole = Role::load('authenticated');
    $authenticatedRole->grantPermission('access content')->save();
    $authenticatedRole->grantPermission('edit any notice content')->save();

    // The edit page should not be accessible, as fallback neutral lets the
    // "edit any article content" permission decide and that permission is not
    // set on the current user:
    $this->drupalGet('/node/2/edit');
    $this->assertSession()->statusCodeEquals(403);

    // Node 1 should be accessible, as it is a notice, and the user has this
    // permission:
    $this->drupalGet('/node/1/edit');
    $this->assertSession()->statusCodeEquals(200);
  }

}
